; Allows extra error checking with modified version
; of ca65. Otherwise acts like a constant of 0.
ADDR = 0

; Switches to Segment and places Line there.
; Line can be an .align directive, .res, .byte, etc.
; Examples:
; seg_data BSS, .align 256
; seg_data RODATA, {message: .byte "Test",0}
.macro seg_data Segment, Line
	.pushseg
	.segment .string(Segment)
		Line
	.popseg
.endmacro

; Reserves Size bytes in Segment for Name.
; If Size is omitted, reserves one byte.
.macro seg_res Segment, Name, Size
	.ifblank Size
		seg_data Segment, Name: .res 1
	.else
		seg_data Segment, Name: .res Size
	.endif
.endmacro

; Shortcuts for zeropage, bss, and stack
.define zp_res  seg_res ZEROPAGE,
.define bss_res seg_res BSS,
.define sp_res  seg_res STACK,

; Copies byte from Src to Addr. If Src begins with #,
; it sets Addr to the immediate value.
; Out: A = byte copied
; Preserved: X, Y
.macro mov Addr, Src
	lda Src
	sta Addr
.endmacro

; Copies word from Src to Addr. If Src begins with #,
; it sets Addr the immediate value.
; Out: A = high byte of word
; Preserved: X, Y
.macro movw Addr, Src
	.if .match( .left( 1, {Src} ), # )
		lda #<(.right( .tcount( {Src} )-1, {Src} ))
		sta Addr
		lda #>(.right( .tcount( {Src} )-1, {Src} ))
		sta 1+(Addr)
	.else
		lda Src
		sta Addr
		lda 1+(Src)
		sta 1+(Addr)
	.endif
.endmacro

; Increments 16-bit value at Addr.
; Out: EQ/NE based on resulting 16-bit value
; Preserved: A, X, Y
.macro incw Addr
	.local @Skip
	inc Addr
	bne @Skip
	inc 1+(Addr)
@Skip:
.endmacro

; Adds Src to word at Addr.
; Out: A = high byte of result, carry set appropriately
; Preserved: X, Y
.macro addw Addr, Src
	lda Addr
	clc
	adc Src
	sta Addr
	
	lda 1+(Addr)
	adc 1+(Src)
	sta 1+(Addr)
.endmacro

; Splits list of words into tables of low and high bytes
; Example: split_words foo, {$1234, $5678}
; expands to:
; foo_l: $34, $78
; foo_h: $12, $56
; foo_count = 2
.macro split_words Label, Words
	.ident (.concat (.string(Label), "_l")): .lobytes Words
	.ident (.concat (.string(Label), "_h")): .hibytes Words
	.ident (.concat (.string(Label), "_count")) = * - .ident (.concat (.string(Label), "_h"))
.endmacro

.ifp02
	; 6502 doesn't define these alternate names
	.define blt bcc
	.define bge bcs
.endif
.define jlt jcc
.define jge jcs

; Jxx Target = Bxx Target, except it can go farther than
; 128 bytes. Implemented via branch around a JMP.

; Don't use ca65's longbranch, because they fail for @labels
;.macpack longbranch

.macro jeq Target
	bne *+5
	jmp Target
.endmacro

.macro jne Target
	beq *+5
	jmp Target
.endmacro

.macro jmi Target
	bpl *+5
	jmp Target
.endmacro

.macro jpl Target
	bmi *+5
	jmp Target
.endmacro

.macro jcs Target
	bcc *+5
	jmp Target
.endmacro

.macro jcc Target
	bcs *+5
	jmp Target
.endmacro

.macro jvs Target
	bvc *+5
	jmp Target
.endmacro

.macro jvc Target
	bvs *+5
	jmp Target
.endmacro
