; Generates tempo synchronized with taps of button

; Public routines:
; init_tempo
; tap_tempo
; update_tempo

; Must be defined externally and incremented each NMI
; bss_res nmi_count,2

; Value added to tempo_accum each 1/60 second
zp_res  tempo_rate,2
zp_res  tempo_accum,2

; Set when tempo becomes stable after taps, cleared every
; few taps; you can also clear
bss_res tempo_stable

; Tapes more than this many frames are considered
; resync taps only.
tap_timeout = 90

; Number of taps after initial tap before new tempo
; is established
beats_to_new_tempo = 4

.include "div32.s"

; Internal
bss_res tempo_prev              ; where tempo_accum loop left off
bss_res tempo_tap_time,2        ; time of tap
zp_res  tempo_tap_count         ; number of taps since update_tempo
bss_res tempo_prev_tap,2        ; time of previous tap
bss_res tempo_new_ref,2         ; time of potential new reference
bss_res tempo_ref,2             ; time of current reference
bss_res tempo_beat_count,2      ; beats since current reference
bss_res tempo_new_beats         ; incremented on each processed tap
bss_res tempo_stable_delay

; Initializes and uses default tempo
init_tempo:
	lda #0
	sta tempo_stable_delay
	sta tempo_prev
	sta tempo_accum
	sta tempo_accum+1
	sta tempo_tap_count
	sta tempo_stable
	
	movw tempo_rate,#1000
	movw tempo_prev_tap,#$FF00
	
	; If NMI occurs while we're reading nmi_count, re-read it
:       lda nmi_count
	ldx nmi_count + 1
	cmp nmi_count
	bne :-
	sta tempo_ref
	stx tempo_ref + 1
	
	rts


; Call when tempo button is tapped. Must be done from inside
; NMI, so that nmi_count will be accurate and unchanging.
; Preserved: X, Y
tap_tempo:
	; Don't update if busy
	bit tempo_tap_count
	bmi :+
	movw tempo_tap_time,nmi_count
	inc tempo_tap_count
:       rts


; Updates tempo based on taps. Should be called at least 10
; times per second.
update_tempo:
	; Handle any new taps
	lda tempo_tap_count
	beq :+
	jsr @handle_tap
	
:       ; A = frames since last update
	lda nmi_count
	tay
	sec
	sbc tempo_prev
	beq @no_frames
	
	; tempo_accum += tempo_rate * A
	; also increment beat_count
	sty tempo_prev
	tay
@loop:  addw tempo_accum,tempo_rate
	bcc :+
	incw tempo_beat_count
	lda tempo_stable_delay
	beq :+
	dec tempo_stable_delay
	bne :+
	sta tempo_stable
:       dey
	bne @loop

@no_frames:
	rts

@handle_tap:
	mov tempo_tap_count,#-1
	mov tempo_stable_delay,#2
	inc tempo_new_beats
	
	; See if tap is within 2 seconds of previous
	lda tempo_tap_time+0
	sec
	sbc tempo_prev_tap+0
	tax
	lda tempo_tap_time+1
	sbc tempo_prev_tap+1
	tay
	movw tempo_prev_tap,tempo_tap_time
	inc tempo_tap_count
	tya
	bne @ref
	cpx #tap_timeout
	blt @recent
	
	; More than timeout, so set new potential ref
@ref:   movw tempo_new_ref,tempo_prev_tap
	; Negative avoids sync
	mov tempo_new_beats,#-beats_to_new_tempo
	
	; Lone tap merely marks as stable
	mov tempo_stable,#1
	rts
	
	; Within 2 seconds, so see if this is beats_to_new_tempo-th tap
@recent:
	; If tap was less than half a beat after
	; current one, don't count it as another beat
	bit tempo_accum + 1
	bpl :+
	incw tempo_beat_count
:       
	lda tempo_new_ref
	ora tempo_new_ref+1
	beq @adjust
	lda tempo_new_beats
	bne @adjust
	
	; beats_to_new_tempo-th tap, so set first as new reference
@fresh:
	movw tempo_beat_count,#beats_to_new_tempo
	movw tempo_ref,tempo_new_ref
	movw tempo_new_ref,#0
	mov tempo_new_beats,#0
	
@adjust:
	; Calculate average tempo_rate from reference
	; to this tap
	
	; divisor = tempo_prev_tap - ref
	lda tempo_prev_tap
	sec
	sbc tempo_ref
	sta divisor
	lda tempo_prev_tap + 1
	sbc tempo_ref + 1
	sta divisor + 1
	
	; dividend = tempo_beat_count*$10000 + divisor/2 (for rounding)
	lsr a
	sta dividend + 1
	lda divisor
	ror a
	sta dividend
	movw dividend+2,tempo_beat_count
	
	jsr div32
	
	; Ignore values outside 30 to 290 BPM
	lda quotient+1
	cmp #21
	bge :+
	cmp #2
	blt :+
	movw tempo_rate,quotient
:
	; tempo_accum = tempo_rate*2
	mov tempo_accum+1,tempo_rate+1
	lda tempo_rate
	asl a
	rol tempo_accum+1
	sta tempo_accum
	
	; Add based on tap time, not where it was last run to.
	; So this basically rewinds time a little bit.
	mov tempo_prev,tempo_prev_tap

	; Show as stable every 4 beats
	mov tempo_stable,#0
	lda tempo_new_beats
	bmi :+
	and #$03
	bne :+
	sta tempo_stable_delay
	inc tempo_stable
:       
	rts
